// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2008  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Apologies for the lack of comments... I never thought this would end up
// online. If you have any questions feel free to email me at:
// ali@arkitus.com
// ------------------------------------------------------------------------------

class ImportantBoxCoordinate
{
  int i;
  int j;
  int k;
  
  ImportantBoxCoordinate( int i, int j, int k )
  {
    this.i = i;
    this.j = j;
    this.k = k;
  }
}

class SceneSolver
{
  Character[] characters;
  
  SceneSolver
  ()
  {
    characters = new Character[2];
    characters[0] = new Character( "a" );
    characters[1] = new Character( "e" );
  }
  
  void setCharacter
  ( Character character,
    int axisIdentifier )
  {
    if ( axisIdentifier == 0 || axisIdentifier == 1 )
    {
      characters[ axisIdentifier % 2 ] = character;
    }
    else
    {
      characters[ axisIdentifier % 2 ] = character.reverse();
    }
  }
  
  Box[] solve
  ( Box[] currentBoxes )
  {
    boolean[][][] solutionSpace;
    ArrayList importantBoxes;
    Box[] r = new Box[ currentBoxes.length ];
    
    solutionSpace = new boolean[ CHARACTER_WIDTH ][ CHARACTER_HEIGHT ][ CHARACTER_WIDTH ];
    
    for ( int i=0; i<CHARACTER_HEIGHT; i++ )
    {
      for ( int j=0; j<CHARACTER_WIDTH; j++ )
      {
        for ( int k=0; k<CHARACTER_WIDTH; k++ )
        {
          solutionSpace[j][i][k] = getCharacterValue( 0, j, i ) && getCharacterValue( 1, j, CHARACTER_WIDTH - k - 1 );
        }
      }
    }
    
    importantBoxes = new ArrayList();
    
    for ( int i=0; i<CHARACTER_HEIGHT; i++ )
    {
      for ( int j=0; j<CHARACTER_WIDTH; j++ )
      {
        for ( int k=0; k<CHARACTER_WIDTH; k++ )
        {
          if ( solutionSpace[j][i][k] )
          {
            ImportantBoxCoordinate coordinate = new ImportantBoxCoordinate( i, j, k );
            importantBoxes.add( coordinate );
          }
        }
      }
    }
    
    for ( int i=0; i<currentBoxes.length; i++ )
    {
      if ( i < importantBoxes.size() )
      {
        ImportantBoxCoordinate coordinate = (ImportantBoxCoordinate) importantBoxes.get(i); 
      
        r[i] = new Box();
        r[i].active = true;
        r[i].position.x = ( coordinate.i - CHARACTER_WIDTH/2 ) * ( 2 * BOX_SIZE + GRID_SPACING ) + GRID_SPACING/2;
        r[i].position.y = ( coordinate.j - CHARACTER_HEIGHT/2 ) * ( 2 * BOX_SIZE + GRID_SPACING ) + GRID_SPACING/2;
        r[i].position.z = ( coordinate.k - CHARACTER_WIDTH/2 ) * ( 2 * BOX_SIZE + GRID_SPACING ) + GRID_SPACING/2;
      }
      else
      {
        if ( currentBoxes[i].position.length() > SCENE_RADIUS * 0.9 )
        {
          r[i] = new Box( currentBoxes[i].position );
        }
        else
        {
          r[i] = new Box( findPositionOnSphere( r ) );
        }
      }
    }
    
    return r;
  }
  
  boolean getCharacterValue
  ( int characterIdentifier,
    int i,
    int j )
  {
    if ( characters[ characterIdentifier ] != null )
    {
      return characters[ characterIdentifier ].map[ i ][ j ];
    }
    
    return true;
  }
}
